// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PrefabLoader.hpp"
#include "SceneLoader.hpp"
#include "game/Game.hpp"
#include "assert/assert.hpp"
#include "asset/loader/TextureLoader.hpp"
#include "scene/Scene.hpp"
#include "component/SceneComponentRegistry.hpp"

using namespace Dawn;

const std::string PrefabLoader::ASSET_TYPE = "prefab";

PrefabLoader::PrefabLoader(
  const std::shared_ptr<AssetManager> assetManager,
  const std::string name
) :
  LoaderForSceneItems(assetManager, name)
{
  this->typetest = this->getAssetType();
}

void PrefabLoader::updateSync() {
  switch(this->state) {
    case PrefabLoaderState::INITIAL:
      jsonLoader = getAssetManager()->get<JSONLoader>(this->name);
      this->state = PrefabLoaderState::LOADING_JSON;
      break;
    
    case PrefabLoaderState::LOADING_JSON:
      assertNotNull(this->jsonLoader, "JSON Loader is NULL?");
      if(!this->jsonLoader->loaded) return;
      this->state = PrefabLoaderState::LOADING_DEPENDENCIES;
      this->setupDependencies();
      break;

    case PrefabLoaderState::LOADING_DEPENDENCIES:
      assertTrue(this->jsonLoader->loaded, "JSON loader not loaded?");

      // Check if all dependencies are loaded.
      for(auto &asset : ctx->assets) {
        if(!asset.second->loaded) return;
      }

      this->state = PrefabLoaderState::DEPENDENCIES_LOADED;
      this->loaded = true;
      break;

    default:
      break;
  }
}

void PrefabLoader::updateAsync() {
}

std::string PrefabLoader::getAssetType() const {
  return PrefabLoader::ASSET_TYPE;
}

void PrefabLoader::stagePrefab(std::shared_ptr<SceneItem> item) {
  assertTrue(this->loaded, "Prefab not loaded");

  // Force-set new context values
  ctx->data = this->jsonLoader->data;
  
  // Load the scene item.
  item->load(ctx);
  
  auto &itemData = this->jsonLoader->data;
  if(itemData.contains("components")) {
    for(auto &cmpItem : itemData["components"].items()) {
      auto &cmpName = cmpItem.key();
      auto &cmpData = cmpItem.value();
      assertTrue(
        cmpData.contains("type"),
        "Component missing type in %s",
        cmpName.c_str()
      );

      auto cmpType = cmpData["type"].get<std::string>();
      auto cmp = SceneComponentRegistry::createComponent(
        cmpType, item
      );
      ctx->data = cmpData;
      ctx->components[cmpName] = cmp;
      cmp->load(ctx);
    }
  }
}

PrefabLoader::~PrefabLoader() {

}